---
title: llarp::vpn::QueuedNetworkInterface
summary: an implementation that uses a lockless queue to feed packets in for io. 

---

# llarp::vpn::QueuedNetworkInterface



an implementation that uses a lockless queue to feed packets in for io.  [More...](#detailed-description)


`#include <vpn_platform.hpp>`

Inherits from [llarp::vpn::NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md), [llarp::vpn::I_Packet_IO](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md)

Inherited by [llarp::apple::AppleVPNInterface](Classes/classllarp_1_1apple_1_1AppleVPNInterface.md)

## Public Functions

|                | Name           |
| -------------- | -------------- |
| | **[QueuedNetworkInterface](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-queuednetworkinterface)**([InterfaceInfo](Classes/structllarp_1_1vpn_1_1InterfaceInfo.md) info, std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > send_wakeup, std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > recv_wakeup)<br>construct using recv_wakeup and send_wakeup that are informed when io happens.  |
| void | **[offer_read](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-offer-read)**(std::vector< [net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) > pkt)<br>thread safe offer many ip packets to be read.  |
| void | **[offer_write](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-offer-write)**(std::vector< [net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) > pkt)<br>thread safe offer an many ip packets to be written.  |
| virtual void | **[Stop](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-stop)**() override<br>stop operation and tear down anything that [Start()](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-start) set up.  |
| virtual int | **[PollFD](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-pollfd)**() const override<br>get pollable fd for reading  |

## Protected Attributes

|                | Name           |
| -------------- | -------------- |
| [thread::Queue](Classes/classllarp_1_1thread_1_1Queue.md)< [net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) > | **[_send](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#variable--send)**  |
| [thread::Queue](Classes/classllarp_1_1thread_1_1Queue.md)< [net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) > | **[_recv](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#variable--recv)**  |
| std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > | **[_send_wakeup](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#variable--send-wakeup)**  |
| std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > | **[_recv_wakeup](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#variable--recv-wakeup)**  |

## Additional inherited members

**Public Functions inherited from [llarp::vpn::NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md)**

|                | Name           |
| -------------- | -------------- |
| | **[NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md#function-networkinterface)**([InterfaceInfo](Classes/structllarp_1_1vpn_1_1InterfaceInfo.md) info) |
| | **[NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md#function-networkinterface)**(const [NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md) & ) =delete |
| | **[NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md#function-networkinterface)**([NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md) && ) =delete |
| const [InterfaceInfo](Classes/structllarp_1_1vpn_1_1InterfaceInfo.md) & | **[Info](Classes/classllarp_1_1vpn_1_1NetworkInterface.md#function-info)**() const |

**Protected Attributes inherited from [llarp::vpn::NetworkInterface](Classes/classllarp_1_1vpn_1_1NetworkInterface.md)**

|                | Name           |
| -------------- | -------------- |
| [InterfaceInfo](Classes/structllarp_1_1vpn_1_1InterfaceInfo.md) | **[m_Info](Classes/classllarp_1_1vpn_1_1NetworkInterface.md#variable-m-info)**  |

**Public Functions inherited from [llarp::vpn::I_Packet_IO](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md)**

|                | Name           |
| -------------- | -------------- |
| virtual | **[~I_Packet_IO](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-~i-packet-io)**() =default |
| virtual void | **[Start](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-start)**()<br>start any platform specific operations before running  |
| virtual [net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) | **[ReadNextPacket](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-readnextpacket)**() =0<br>read next ip packet, return an empty packet if there are none ready.  |
| virtual bool | **[WritePacket](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-writepacket)**([net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) pkt) =0<br>write a packet to the interface returns false if we dropped it  |


## Detailed Description

```cpp
class llarp::vpn::QueuedNetworkInterface;
```

an implementation that uses a lockless queue to feed packets in for io. 

this provides a reactor style io patter for platforms with a proactor style event loop. 

## Public Functions Documentation

### function QueuedNetworkInterface

```cpp
QueuedNetworkInterface(
    InterfaceInfo info,
    std::shared_ptr< EventLoopWakeup > send_wakeup,
    std::shared_ptr< EventLoopWakeup > recv_wakeup
)
```

construct using recv_wakeup and send_wakeup that are informed when io happens. 

send_wakeup is woken up when we queue ip packets for write with [offer_write()](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-offer-write). recv_wakeup is woken up when we queue ip packets for read with [offer_read()](Classes/classllarp_1_1vpn_1_1QueuedNetworkInterface.md#function-offer-read). 


### function offer_read

```cpp
void offer_read(
    std::vector< net::IPPacket > pkt
)
```

thread safe offer many ip packets to be read. 

wakeup on recv_wakeup is triggered regardless of the number of packets. 


### function offer_write

```cpp
void offer_write(
    std::vector< net::IPPacket > pkt
)
```

thread safe offer an many ip packets to be written. 

wakeup on send_wakeup is triggered regardless of the number of packets. 


### function Stop

```cpp
virtual void Stop() override
```

stop operation and tear down anything that [Start()](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-start) set up. 

**Reimplements**: [llarp::vpn::I_Packet_IO::Stop](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-stop)


### function PollFD

```cpp
virtual int PollFD() const override
```

get pollable fd for reading 

**Reimplements**: [llarp::vpn::I_Packet_IO::PollFD](Classes/classllarp_1_1vpn_1_1I__Packet__IO.md#function-pollfd)


## Protected Attributes Documentation

### variable _send

```cpp
thread::Queue< net::IPPacket > _send;
```


### variable _recv

```cpp
thread::Queue< net::IPPacket > _recv;
```


### variable _send_wakeup

```cpp
std::shared_ptr< EventLoopWakeup > _send_wakeup;
```


### variable _recv_wakeup

```cpp
std::shared_ptr< EventLoopWakeup > _recv_wakeup;
```


-------------------------------

Updated on 2023-03-25 at 01:02:44 +0000